/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2021. All rights reserved.
 */

package com.huawei.agc.flutter.applinking.utils;

import com.huawei.agconnect.applinking.AppLinkingException;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import io.flutter.Log;

public class AGCAppLinkingException extends Exception {
    private final String errorCode;
    private final String errorMessage;
    private final Map<String, Object> errorDetails = new HashMap<>();

    public AGCAppLinkingException(String code, String message) {
        this(code, message, "PluginException");
    }

    private AGCAppLinkingException(String code, String message, String exceptionType) {
        this.errorCode = code;
        this.errorMessage = message;
        this.errorDetails.put("exceptionType", exceptionType);

        Log.e("AGCAppLinkingException", String.format(Locale.ENGLISH, "(%s) %s: %s", exceptionType, this.errorCode, this.errorMessage));
    }

    public static AGCAppLinkingException from(Exception e) {
        if (e instanceof AGCAppLinkingException) {
            return (AGCAppLinkingException) e;
        } else {
            if (e instanceof AppLinkingException) {
                return new AGCAppLinkingException("UNKNOWN", e.getMessage(), "SDKException");
            } else {
                return new AGCAppLinkingException("UNKNOWN", e.getMessage());
            }
        }
    }

    public String getErrorCode() {
        return this.errorCode;
    }

    public String getErrorMessage() {
        return this.errorMessage;
    }

    public Map<String, Object> getErrorDetails() {
        return this.errorDetails;
    }
}
